require_relative 'minitest_helper'
require 'ostruct'

module Qdsl
  class ColumnTest < Minitest::Test
    def test_convert_from_column
      column = Column.new('name', 'alias_name', 'source')
      result = Column[column]
      assert_equal column, result
    end

    def test_convert_from_string
      result = Column['name']
      assert_equal 'name', result.name
      assert_nil result.alias_name
      assert_nil result.source
    end

    def test_convert_from_symbol
      result = Column[:name]
      assert_equal 'name', result.name
      assert_nil result.alias_name
      assert_nil result.source
    end

    def test_initialize_strings
      result = Column.new('name', 'alias_name', 'source')
      assert_equal 'name', result.name
      assert_equal 'alias_name', result.alias_name
      assert_equal 'source', result.source
    end

    def test_initialize_symbols
      result = Column.new(:name, :alias_name, :source)
      assert_equal 'name', result.name
      assert_equal 'alias_name', result.alias_name
      assert_equal :source, result.source
    end

    def test_initialize_name_only_string
      result = Column.new('name')
      assert_equal 'name', result.name
      assert_nil result.alias_name
      assert_nil result.source
    end

    def test_initialize_name_only_symbol
      result = Column.new(:name)
      assert_equal 'name', result.name
      assert_nil result.alias_name
      assert_nil result.source
    end

    def test_with_name_string
      result = Column.new('name', 'alias_name', 'source').with_name('new_name')
      assert_equal 'new_name', result.name
      assert_equal 'alias_name', result.alias_name
      assert_equal 'source', result.source
    end

    def test_with_name_symbol
      result = Column.new('name', 'alias_name', 'source').with_name(:new_name)
      assert_equal 'new_name', result.name
      assert_equal 'alias_name', result.alias_name
      assert_equal 'source', result.source
    end

    def test_with_alias_name_string
      result = Column.new('name', 'alias_name', 'source').with_alias_name('new_alias_name')
      assert_equal 'name', result.name
      assert_equal 'new_alias_name', result.alias_name
      assert_equal 'source', result.source
    end

    def test_with_alias_name_symbol
      result = Column.new('name', 'alias_name', 'source').with_alias_name(:new_alias_name)
      assert_equal 'name', result.name
      assert_equal 'new_alias_name', result.alias_name
      assert_equal 'source', result.source
    end

    def test_with_source
      result = Column.new('name', 'alias_name', 'source').with_source('new_source')
      assert_equal 'name', result.name
      assert_equal 'alias_name', result.alias_name
      assert_equal 'new_source', result.source
    end

    def test_to_expression
      result = Column.new('name', 'alias_name', 'source').to_expression
      assert result.is_a?(IsTrue)
    end

    def test_true
      result = Column.new('name', 'alias_name', 'source').true?
      assert result.is_a?(IsTrue)
    end

    def test_as_string
      result = Column.new('name', 'alias_name', 'source').as('new_name_alias')
      assert_equal 'name', result.name
      assert_equal 'new_name_alias', result.alias_name
      assert_equal 'source', result.source
    end

    def test_as_symbol
      result = Column.new('name', 'alias_name', 'source').as(:new_name_alias)
      assert_equal 'name', result.name
      assert_equal 'new_name_alias', result.alias_name
      assert_equal 'source', result.source
    end

    def test_render_sql_name_only
      column = Column.new('name').with_source(OpenStruct.new(id: '__id__'))
      result = column.render_sql(nil, nil)
      assert_equal '__id__.name', result.sql
    end

    def test_render_sql_alias_name
      column = Column.new('name', 'alias_name').with_source(OpenStruct.new(id: '__id__'))
      result = column.render_sql(nil, nil)
      assert_equal '__id__.name AS alias_name', result.sql
    end
  end
end

