require_relative 'minitest_helper'

module Qdsl
  class ExpressionTest < QueryTest
    def test_simple_and
      query = Qdsl::select(:id).
        from(@clients).
        where { |t| t.first_name.equals('Richard').and(t.last_name.equals('Cook')) }

      result = query.render

      assert_sql <<-SQL, result.sql
SELECT
  _00.id
FROM clients AS _00
WHERE (_00.first_name = :_param00) AND (_00.last_name = :_param01)
      SQL
      assert_equal 2, result.parameters.size
      assert_equal 'Richard', result.parameters['_param00']
      assert_equal 'Cook', result.parameters['_param01']
    end

    def test_multiple_and
      query = Qdsl::select(:id).
        from(@clients).
        where { |t| t.first_name.equals('Richard').and(t.last_name.equals('Cook')).and(t.id.equals('ID')) }

      result = query.render

      assert_sql <<-SQL, result.sql
SELECT
  _00.id
FROM clients AS _00
WHERE (_00.first_name = :_param00) AND (_00.last_name = :_param01) AND (_00.id = :_param02)
      SQL
      assert_equal 3, result.parameters.size
      assert_equal 'Richard', result.parameters['_param00']
      assert_equal 'Cook', result.parameters['_param01']
      assert_equal 'ID', result.parameters['_param02']
    end

    def test_simple_or
      query = Qdsl::select(:id).
        from(@clients).
        where { |t| t.first_name.equals('Richard').or(t.last_name.equals('Cook')) }

      result = query.render

      assert_sql <<-SQL, result.sql
SELECT
  _00.id
FROM clients AS _00
WHERE (_00.first_name = :_param00) OR (_00.last_name = :_param01)
      SQL
      assert_equal 2, result.parameters.size
      assert_equal 'Richard', result.parameters['_param00']
      assert_equal 'Cook', result.parameters['_param01']
    end

    def test_multiple_or
      query = Qdsl::select(:id).
        from(@clients).
        where { |t| t.first_name.equals('Richard').or(t.last_name.equals('Cook')).or(t.id.equals('ID')) }

      result = query.render

      assert_sql <<-SQL, result.sql
SELECT
  _00.id
FROM clients AS _00
WHERE (_00.first_name = :_param00) OR (_00.last_name = :_param01) OR (_00.id = :_param02)
      SQL
      assert_equal 3, result.parameters.size
      assert_equal 'Richard', result.parameters['_param00']
      assert_equal 'Cook', result.parameters['_param01']
      assert_equal 'ID', result.parameters['_param02']
    end

    def test_multiple_or2
      query = Qdsl::select(:id).
        from(@clients).
        where { |t| big_or([:value0, :value1, :value2, :value3], t) }

      result = query.render

      assert_sql <<-SQL, result.sql
SELECT
  _00.id
FROM clients AS _00
WHERE (_00.value0 = TRUE) OR (_00.value1 = TRUE) OR (_00.value2 = TRUE) OR (_00.value3 = TRUE)
      SQL
    end

    def test_boolean
      query = Qdsl::select(:id).
        from(@clients).
        where { |t| t.first_name.equals('Richard').or(t.last_name.equals('Cook')).and(t.id.equals('ID').or(t.id.equals('OTHER_ID'))) }

      result = query.render

      assert_sql <<-SQL, result.sql
SELECT
  _00.id
FROM clients AS _00
WHERE ((_00.first_name = :_param00)
  OR (_00.last_name = :_param01))
  AND ((_00.id = :_param02)
  OR (_00.id = :_param03))
      SQL
      assert_equal 4, result.parameters.size
      assert_equal 'Richard', result.parameters['_param00']
      assert_equal 'Cook', result.parameters['_param01']
      assert_equal 'ID', result.parameters['_param02']
      assert_equal 'OTHER_ID', result.parameters['_param03']
    end

    private

    def big_or(ids, t)
      ids.collect { |x| t[x].equals(true) }.inject { |acc, x| acc.or(x) }
    end
  end
end

