require_relative 'minitest_helper'

module Qdsl
  class InnerJoinTest < QueryTest
    def test_inner_join_simple
      query = Qdsl::select { |t0, t1| [t0.id, t1.member_id] }.
        from(@clients).
        inner_join(@members).on { |f, j| f.id.equals(j.id) }

      result = query.render

      assert_sql <<-SQL, result.sql
SELECT
  _00.id,
  _01.member_id
FROM clients AS _00
INNER JOIN members AS _01 ON _00.id = _01.id
      SQL
    end

    def test_inner_join
      query = Qdsl::select { |t0, t1| [t0.id, t1.member_id] }.
        from(@clients).
        inner_join(@members).on { |t0, t1| t0.id.equals(t1.first_name) }

      result = query.render

      assert_sql <<-SQL, result.sql
SELECT
  _00.id,
  _01.member_id
FROM clients AS _00
INNER JOIN members AS _01 ON _00.id = _01.first_name
      SQL
    end

    def test_multiple_inner_joins
      query = Qdsl::select { |t0, t1, t2| [t0.id, t1.member_id, t2.name] }.
        from(@clients).
        inner_join(@members).on { |f, j| f.id.equals(j.first_name) }.
        inner_join(@colours).on { |f, j| f.first_name.equals(j.name) }

      result = query.render

      assert_sql <<-SQL, result.sql
SELECT
  _00.id,
  _01.member_id,
  _02.name
FROM clients AS _00
INNER JOIN members AS _01 ON _00.id = _01.first_name
INNER JOIN colours AS _02 ON _00.first_name = _02.name
      SQL
    end

    def test_multiple_inner_join_same
      query = Qdsl::select { |t0, t1| [t0.id, t1.id] }.
        from(@clients).
        inner_join(@clients).on { |f, j| f.id.equals(j.first_name) }

      result = query.render

      assert_sql <<-SQL, result.sql
SELECT
  _00.id,
  _01.id
FROM clients AS _00
INNER JOIN clients AS _01 ON _00.id = _01.first_name
      SQL
    end

    def test_multiple_inner_join_same_where
      query = Qdsl::select { |t0, t1, t2| [t0.id, t1.id] }.
        from(@clients).
        inner_join(@clients).on { |f, j| f.id.equals(j.first_name) }.
        inner_join(@clients).on { |f, j| f.id.equals(j.first_name) }.
        where { |t0, t1, t2| t0.id.equals(t1.id) }

      result = query.render

      assert_sql <<-SQL, result.sql
SELECT
  _00.id,
  _01.id
FROM clients AS _00
INNER JOIN clients AS _01 ON _00.id = _01.first_name
INNER JOIN clients AS _02 ON _00.id = _02.first_name
WHERE _00.id = _01.id
       SQL
    end
  end
end

