require_relative 'minitest_helper'

module Qdsl
  class IsTrueTest < QueryTest
    def test_explicit
      query = Qdsl::select(:id).from(@clients).where { |t| t.id.true? }
      result = query.render

      assert_sql <<-SQL, result.sql
SELECT
  _00.id
FROM clients AS _00
WHERE _00.id
      SQL
    end

    def test_implicit_method
      query = Qdsl::select(:id).from(@clients).where { |t| t.id }
      result = query.render

      assert_sql <<-SQL, result.sql
SELECT
  _00.id
FROM clients AS _00
WHERE _00.id
      SQL
    end

    def test_implicit_string
      query = Qdsl::select(:id).from(@clients).where { |t| t['id'] }
      result = query.render

      assert_sql <<-SQL, result.sql
SELECT
  _00.id
FROM clients AS _00
WHERE _00.id
      SQL
    end

    def test_implicit_symbol
      query = Qdsl::select(:id).from(@clients).where { |t| t[:id] }
      result = query.render

      assert_sql <<-SQL, result.sql
SELECT
  _00.id
FROM clients AS _00
WHERE _00.id
      SQL
    end

    def test_operators
      query = Qdsl::select(:id).from(@clients).where do |t|
        t.id.true?.and(t.id.true?.or(t.id.true?))
      end
      result = query.render

      assert_sql <<-SQL, result.sql
SELECT
  _00.id
FROM clients AS _00
WHERE (_00.id) AND ((_00.id) OR (_00.id))
      SQL
    end
  end
end

